# IoC Container for typescript
This is a lightweight annotation-based dependency injection container for typescript.

It can be used on browser or on node.js server code.

It is under development and is not safe for production yet. 

## Basic Usage

```javascript
import Resource = IoC.Resource;
import Inject = IoC.Inject;

@Resource
class PersonDAO {
    @Inject
    restProxy: PersonRestProxy;
}
```

That's it. You can just call now:

```javascript
let personDAO: PersonDAO = new PersonDAO();
```

And the dependencies will be resolved.

You can also inject constructor parameters, like:

```javascript
@IoC.Resource
class PersonService {
	private personDAO: PersonDAO;
	constructor( @Inject personDAO: PersonDAO ) {
		this.personDAO = personDAO;
    }
}
```

and then, if you make an injection to this class, like...

```javascript
@IoC.Resource
class PersonController {
	@Inject
	private personService: PersonService;
}
```

The container will create an instance of PersonService that receives the PersonDAO from the container on its constructor.
But you still can call:

```javascript
let personService: PersonService = new PersonService(myPersonDAO);
```

And pass your own instance of PersonDAO to PersonService.

Note that any type that have a constructor can be injected, not only the types that you decorate with @IoC.Resource.

```javascript
@IoC.Resource
class PersonController {
	@Inject
	private personService: PersonService;

	@Inject
	creationTime: Date;
}
```

## Inheritance

You don't have to do anything special to work with sub-types.

```javascript
@IoC.Resource
class abstract BaseDAO {

	@Inject
	creationTime: Date;
}

@IoC.Resource
class PersonDAO extends BaseDAO {

	@Inject
	private personRestProxy: PersonRestProxy;
}

@IoC.Resource
class ProgrammerDAO extends PersonDAO {

	@Inject
	private programmerRestProxy: PersonRestProxy;
}
```

Will work as expected.

## Scopes

You can use scopes to manage your instances as:

```javascript
@IoC.Resource ({scope: Scope.Singleton}) 
class PersonService {
	@Inject
	private personDAO: PersonDAO;
}

@IoC.Resource
class PersonController {
	@Inject
	private personService: PersonService;

	@Inject
	creationTime: Date;
}
```

So we can create a lot of PersonController instances, but all of them will share the same singleton instance of PersonService

```javascript
let controller1: PersonController = new PersonController();
let controller2: PersonController = new PersonController();

```

We have two pre defined scopes (Scope.Singleton and Scope.Local), but you can define your own custom Scope. You just have to extends the Scope abstract class;

```javascript
class MyScope extends Scope
{ 
    resolve(iocProvider:Provider, source:any)
    {
        alert('created by my custom scope.')
        return iocProvider();
    }
}
@IoC.Resource ({scope: MyScope}) 
class PersonService {
	@Inject
	private personDAO: PersonDAO;
}
```

## Providers

Providers can be used as a factory for instances created by the IoC Container.

```javascript
let personProvider: IoC.Provider = () => { return new PersonService(); }
@IoC.Resource ({provider: personProvider, scope: MyScope}) 
class PersonService {
	@Inject
	private personDAO: PersonDAO;
}
```

TODO: AsyncProviders are under development...using Promises

## Replacing Implementation class
It is possible to tell the container to replace one class implementation by other sub-class of it. It works even if the sub-type is not a resource itself.

```javascript
const NEW_TYPE_NAME : string = 'ProgrammerDAO';
@IoC.Resource({bindTypeName: NEW_TYPE_NAME})
class PersonDAO extends BaseDAO {

	@Inject
	private personRestProxy: PersonRestProxy;
}

@IoC.Resource
class ProgrammerDAO extends PersonDAO {

	@Inject
	private programmerRestProxy: PersonRestProxy;
}
```

So, everywhere you inject a PersonDAO will receive a ProgrammerDAO instance instead. However, is still possible to create PersonDAO instances through its constructor, like:

```javascript
let personDAO: PersonDAO = new PersonDAO(); // a personDAO instance will be returned, with its dependecies resolved by container
```

## Direct Container interation 
Our intention is configure everything only using annotations, but you can interact direct with the IoC Container:

```javascript
// it will override any annotation configuration
Container.bind(PersonDAO).to(ProgrammerDAO).scope(Scope.Local); 

// that will make any injection to Date to return 
// the same instance, created when the first call is executed.
Container.bind(Date).to(Date).scope(Scope.Singleton); 
```

## Restrictions
- Circular injections are not supported
- You can only inject types that are previously defined into your file. It can be solved by a @LazyInject on future releases
- You can still create new instances of classes in singleton scopes. The singleton instance is returned only requested to the Container, while processing @Inject annotations... It is possible (I am not sure it is good) to change the Object constructor to throw a TypeError when you call directly new MySingletonType(); Accept suggestions on this.
